@@ -47,7 +47,7 @@ namespace netxs::console
                     : inside{ faux },
                       seized{ faux }
                 { }
-                operator bool(){ return inside || seized || region.size; }
+                operator bool () { return inside || seized || region.size; }
                 auto grab(twod const& coord, bool resume)
                 {
                     if (inside)
@@ -119,17 +119,17 @@ namespace netxs::console
                         }
                     });
                 };
-                boss.SUBMIT_T(tier::release, hids::events::mouse::button::click::any, memo, gear)
+                boss.SUBMIT_T(tier::release, hids::events::mouse::button::click::left, memo, gear)
                 {
                     auto& item = items.take(gear);
                     if (item.region.size)
                     {
-                        if (gear.meta(hids::ANYCTRL)) item.region.size = gear.coord - item.region.coor;
+                        if (gear.meta(hids::anyCtrl)) item.region.size = gear.coord - item.region.coor;
                         else                          item.region.size = dot_00;
                     }
                     recalc();
                 };
-                boss.SUBMIT_T(tier::release, hids::events::mouse::button::dblclick::any, memo, gear)
+                boss.SUBMIT_T(tier::release, hids::events::mouse::button::dblclick::left, memo, gear)
                 {
                     auto& item = items.take(gear);
                     auto area = boss.size();
@@ -162,7 +162,7 @@ namespace netxs::console
             }
             void recalc()
             {
-                text data;
+                auto data = text{};
                 auto step = twod{ 5, 1 };
                 auto size = boss.size();
                 size.x += boss.oversz.r;
@@ -190,7 +190,7 @@ namespace netxs::console
                     data = " =SUM(" + ansi::fgc(bluedk).add(data).fgc(blacklt).add(")");
                 }
                 else data = " =SUM(" + ansi::itc(true).fgc(reddk).add("select cells by dragging").itc(faux).fgc(blacklt).add(")");
-                log("calc: DATA ", data);                        
+                log("calc: DATA ", data, ansi::nil());
                 boss.SIGNAL(tier::release, e2::data::text, data);
             }
             // pro::cell_highlight: Configuring the mouse button to operate.
@@ -205,8 +205,10 @@ namespace netxs::console
                 };
                 boss.SUBMIT_T(tier::release, e2::form::drag::start::_<BUTTON>, memo, gear)
                 {
-                    if (items.take(gear).grab(gear.coord, gear.meta(hids::ANYCTRL)))
+                    if (items.take(gear).grab(gear.coord, gear.meta(hids::anyCtrl)))
+                    {
                         gear.dismiss();
+                    }
                 };
                 boss.SUBMIT_T(tier::release, e2::form::drag::pull::_<BUTTON>, memo, gear)
                 {
@@ -235,10 +237,10 @@ namespace netxs::console
 namespace netxs::app::calc
 {
     using events = ::netxs::events::userland::calc;
-    
+
     namespace
     {
-        auto get_text = []()
+        auto get_text = []
         {
             static text cellatix_rows;
             static text cellatix_cols;
@@ -247,39 +249,39 @@ namespace netxs::app::calc
             if (cellatix_text.empty())
             {
                 auto step = 0x030303;
-                auto topclr = 0xffffff;
+                auto topclr = 0xFFffffff;
 
                 auto corner = topclr - 0x1f1f1f;
-                //text cellatix_text_head = ansi::bgc(0xffffff - 0x1f1f1f).fgc(0) + "    ";
-                auto cellatix_text_head = ansi::bgc(corner).fgc(0);
+                //text cellatix_text_head = ansi::bgc(0xFFffffff - 0x1f1f1f).fgc(0) + "    ";
+                auto cellatix_text_head = ansi::bgc(corner).fgc(0xFF000000);
                 for (auto c = 'A'; c <= 'Z'; c++)
                 {
                     auto clr = topclr - 0x0f0f0f;
-                    cellatix_text_head.bgc(clr - step * 0 /* 0xf0f0f0 */).add(" ")
-                                      .bgc(clr - step * 1 /* 0xededed */).add(" ")
-                                      .bgc(clr - step * 2 /* 0xeaeaea */).add(c)
-                                      .bgc(clr - step * 3 /* 0xe7e7e7 */).add(" ")
-                                      .bgc(clr - step * 4 /* 0xe4e4e4 */).add(" ");
+                    cellatix_text_head.bgc(clr - step * 0 /* 0xFFf0f0f0 */).add(" ")
+                                      .bgc(clr - step * 1 /* 0xFFededed */).add(" ")
+                                      .bgc(clr - step * 2 /* 0xFFeaeaea */).add(c)
+                                      .bgc(clr - step * 3 /* 0xFFe7e7e7 */).add(" ")
+                                      .bgc(clr - step * 4 /* 0xFFe4e4e4 */).add(" ");
                 }
                 auto clr = topclr;
-                auto cellatix_text_01 = ansi::bgc(clr - step * 0 /* 0xffffff */).add(" ")
-                                             .bgc(clr - step * 1 /* 0xfcfcfc */).add(" ")
-                                             .bgc(clr - step * 2 /* 0xf9f9f9 */).add(" ")
-                                             .bgc(clr - step * 3 /* 0xf6f6f6 */).add(" ")
-                                             .bgc(clr - step * 4 /* 0xf3f3f3 */).add(" ");
-                auto cellatix_text_00 = ansi::bgc(clr - step * 4 /* 0xf3f3f3 */).add(" ")
-                                             .bgc(clr - step * 3 /* 0xf6f6f6 */).add(" ")
-                                             .bgc(clr - step * 2 /* 0xf9f9f9 */).add(" ")
-                                             .bgc(clr - step * 1 /* 0xfcfcfc */).add(" ")
-                                             .bgc(clr - step * 0 /* 0xffffff */).add(" ");
+                auto cellatix_text_01 = ansi::bgc(clr - step * 0 /* 0xFFffffff */).add(" ")
+                                             .bgc(clr - step * 1 /* 0xFFfcfcfc */).add(" ")
+                                             .bgc(clr - step * 2 /* 0xFFf9f9f9 */).add(" ")
+                                             .bgc(clr - step * 3 /* 0xFFf6f6f6 */).add(" ")
+                                             .bgc(clr - step * 4 /* 0xFFf3f3f3 */).add(" ");
+                auto cellatix_text_00 = ansi::bgc(clr - step * 4 /* 0xFFf3f3f3 */).add(" ")
+                                             .bgc(clr - step * 3 /* 0xFFf6f6f6 */).add(" ")
+                                             .bgc(clr - step * 2 /* 0xFFf9f9f9 */).add(" ")
+                                             .bgc(clr - step * 1 /* 0xFFfcfcfc */).add(" ")
+                                             .bgc(clr - step * 0 /* 0xFFffffff */).add(" ");
                 cellatix_cols = ansi::nil().wrp(wrap::off)
                     + cellatix_text_head;
                 cellatix_text = ansi::nil().wrp(wrap::off);
                 cellatix_rows = ansi::nil().wrp(wrap::off).fgc(blackdk);
                 auto base = topclr - 0x1f1f1f;// 0xe0e0e0;// 0xe4e4e4;
-                auto c1 = ansi::bgc(base); //ansi::bgc(0xf0f0f0);
+                auto c1 = ansi::bgc(base); //ansi::bgc(0xFFf0f0f0);
                 auto c2 = ansi::bgc(base);
-                for (int i = 1; i < 100; i++)
+                for (auto i = 1; i < 100; i++)
                 {
                     auto label = utf::adjust(std::to_string(i), 3, " ", true) + " ";
                     if (!(i % 2))
@@ -292,11 +294,11 @@ namespace netxs::app::calc
                         }
                         cellatix_rows += (i == 99 ? ""s : "\n"s);
                         cellatix_text += utf::repeat(cellatix_text_01, 26)
-                                    + (i == 99 ? ""s : "\n"s);
+                                      + (i == 99 ? ""s : "\n"s);
                     }
                     else
                     {
-                        auto c0 = base + step * (iota)label.length();
+                        auto c0 = base + step * (si32)label.length();
                         for (auto i = 0; i < label.length(); i++)
                         {
                             cellatix_rows += ansi::bgc(c0) + label[i];
@@ -304,16 +306,19 @@ namespace netxs::app::calc
                         }
                         cellatix_rows += (i == 99 ? ""s : "\n"s);
                         cellatix_text += utf::repeat(cellatix_text_00, 26)
-                                    + (i == 99 ? ""s : "\n"s);
+                                      + (i == 99 ? ""s : "\n"s);
                     }
                 }
             }
             return std::tuple{ cellatix_rows, cellatix_cols, cellatix_text };
         };
-        auto build = [](view v)
+        auto build = [](text cwd, text arg, xml::settings& config)
         {
-            const static auto c7 = app::shared::c7;
-            const static auto c3 = app::shared::c3;
+            auto highlight_color = skin::color(tone::highlight);
+            auto label_color     = skin::color(tone::label);
+            auto c3 = highlight_color;
+            auto x3 = cell{ c3 }.alpha(0x00);
+            auto c7 = label_color;
 
             auto [cellatix_rows, cellatix_cols, cellatix_text] = get_text();
 
@@ -327,16 +332,21 @@ namespace netxs::app::calc
                   ->invoke([&](auto& boss)
                   {
                       boss.keybd.accept(true);
+                      boss.SUBMIT(tier::anycast, e2::form::quit, item)
+                      {
+                          boss.base::template riseup<tier::release>(e2::form::quit, item);
+                      };
                       boss.SUBMIT(tier::release, e2::form::upon::vtree::attached, parent)
                       {
-                          static iota i = 0; i++;
+                          static auto i = 0; i++;
                           auto title = ansi::jet(bias::right).add("Spreadsheet\n ~/Untitled ", i, ".ods");
-                          boss.base::template riseup<tier::preview>(e2::form::prop::header, title);
+                          boss.base::template riseup<tier::preview>(e2::form::prop::ui::header, title);
                       };
                   });
+            auto fader = skin::timeout(tone::fader);
             auto object = window->attach(ui::fork::ctor(axis::Y))
                                 ->colors(whitelt, 0);
-                auto menu = object->attach(slot::_1, app::shared::main_menu());
+                auto menu = object->attach(slot::_1, app::shared::main_menu(config));
                 auto all_rail = object->attach(slot::_2, ui::rail::ctor());
                 auto all_stat = all_rail->attach(ui::fork::ctor(axis::Y))
                                         ->plugin<pro::limit>(twod{ -1,-1 },twod{ 136,102 });
@@ -345,18 +355,18 @@ namespace netxs::app::calc
                             auto func_line = func_body->attach(slot::_1, ui::fork::ctor());
                                 auto fx_sum = func_line->attach(slot::_1, ui::fork::ctor());
                                     auto fx = fx_sum->attach(slot::_1, ui::post::ctor())
-                                                    ->plugin<pro::fader>(c7, c3, 150ms)
+                                                    ->plugin<pro::fader>(c7, c3, fader)
                                                     ->plugin<pro::limit>(twod{ 3,-1 }, twod{ 4,-1 })
                                                     ->upload(ansi::wrp(wrap::off).add(" Fx "));
                                 auto ellipsis = func_line->attach(slot::_2, ui::post::ctor())
-                                                         ->plugin<pro::fader>(c7, c3, 150ms)
+                                                         ->plugin<pro::fader>(c7, c3, fader)
                                                          ->plugin<pro::limit>(twod{ -1,1 }, twod{ 3,-1 })
                                                          ->upload(ansi::wrp(wrap::off).add(" … "));
                             auto body_area = func_body->attach(slot::_2, ui::fork::ctor(axis::Y));
                                 auto corner_cols = body_area->attach(slot::_1, ui::fork::ctor());
                                     auto corner = corner_cols->attach(slot::_1, ui::post::ctor())
                                                              ->plugin<pro::limit>(twod{ 4,1 }, twod{ 4,1 })
-                                                             ->upload(ansi::bgc(0xffffff - 0x1f1f1f).fgc(0).add("    "));
+                                                             ->upload(ansi::bgc(0xFFffffff - 0x1f1f1f).fgc(0xFF000000).add("    "));
                                 auto rows_body = body_area->attach(slot::_2, ui::fork::ctor());
                                     auto layers = rows_body->attach(slot::_2, ui::cake::ctor());
                                     auto scroll = layers->attach(ui::rail::ctor())
@@ -396,7 +406,7 @@ namespace netxs::app::calc
                                                        .bgc(whitelt).fgc(blackdk).add(" Sheet1 "));
                             auto plus_pad = sheet_plus->attach(slot::_2, ui::fork::ctor());
                                 auto plus = plus_pad->attach(slot::_1, ui::post::ctor())
-                                                    ->plugin<pro::fader>(c7, c3, 150ms)
+                                                    ->plugin<pro::fader>(c7, c3, fader)
                                                     ->plugin<pro::limit>(twod{ 3,-1 }, twod{ 3,-1 })
                                                     ->upload(ansi::wrp(wrap::off).add(" + "));
                                 auto pad = plus_pad->attach(slot::_2, ui::mock::ctor())
@@ -406,7 +416,7 @@ namespace netxs::app::calc
         };
     }
 
-    app::shared::initialize builder{ "Calc", build };
+    app::shared::initialize builder{ "calc", build };
 }
 
 #endif // NETXS_APP_CALC_HPP
\ No newline at end of file
