@@ -24,18 +24,24 @@ namespace netxs::events::userland
 namespace netxs::app::desk
 {
     using events = ::netxs::events::userland::desk;
-    
+
     namespace
     {
         auto app_template = [](auto& data_src, auto const& utf8)
         {
-            const static auto c4 = app::shared::c4;
-            const static auto x4 = app::shared::x4;
-            const static auto c5 = app::shared::c5;
-            const static auto x5 = app::shared::x5;
-
+            auto danger_color    = skin::color(tone::danger);
+            auto highlight_color = skin::color(tone::highlight);
+            auto c4 = cell{}.bgc(highlight_color.bgc());
+            auto x4 = cell{ c4 }.bga(0x00);
+            auto c5 = danger_color;
+            auto x5 = cell{ c5 }.alpha(0x00);
+            auto fastfader = skin::timeout(tone::fastfader);
+            auto fader = skin::timeout(tone::fader);
             auto item_area = ui::pads::ctor(dent{ 1,0,1,0 }, dent{ 0,0,0,1 })
-                    ->plugin<pro::fader>(x4, c4, 0ms)//150ms)
+                    ->plugin<pro::fader>(x4, c4, fastfader)
+                    ->plugin<pro::notes>(" Running instance:                          \n"
+                                         "   Left click to go to running instance     \n"
+                                         "   Right click to pull the running instance ")
                     ->invoke([&](auto& boss)
                     {
                         auto data_src_shadow = ptr::shadow(data_src);
@@ -47,7 +53,7 @@ namespace netxs::app::desk
                                 inst.SIGNAL(tier::preview, e2::form::layout::expose, inst);
                                 auto& area = inst.base::area();
                                 auto center = area.coor + (area.size / 2);
-                                bell::getref(gear.id)->SIGNAL(tier::release, e2::form::layout::shift, center);  // Goto to the window.
+                                gear.owner.SIGNAL(tier::release, e2::form::layout::shift, center);  // Goto to the window.
                                 gear.pass_kb_focus(inst);
                                 gear.dismiss();
                             }
@@ -76,11 +82,12 @@ namespace netxs::app::desk
                 auto label_area = item_area->attach(ui::fork::ctor());
                     auto mark_app = label_area->attach(slot::_1, ui::fork::ctor());
                         auto mark = mark_app->attach(slot::_1, ui::pads::ctor(dent{ 2,1,0,0 }, dent{ 0,0,0,0 }))
-                                            ->attach(ui::item::ctor(ansi::fgc4(0xFF00ff00).add("‣"), faux));
+                                            ->attach(ui::item::ctor(ansi::fgx(0xFF00ff00).add("‣"), faux));
                         auto app_label = mark_app->attach(slot::_2,
                                     ui::item::ctor(ansi::fgc(whitelt).add(utf8).mgl(0).wrp(wrap::off).jet(bias::left), true, true));
                     auto app_close_area = label_area->attach(slot::_2, ui::pads::ctor(dent{ 0,0,0,0 }, dent{ 0,0,1,1 }))
-                                                    ->template plugin<pro::fader>(x5, c5, 150ms)
+                                                    ->template plugin<pro::fader>(x5, c5, fader)
+                                                    ->template plugin<pro::notes>(" Close instance ")
                                                     ->invoke([&](auto& boss)
                                                     {
                                                         auto data_src_shadow = ptr::shadow(data_src);
@@ -88,7 +95,7 @@ namespace netxs::app::desk
                                                         {
                                                             if (auto data_src = data_src_shadow.lock())
                                                             {
-                                                                data_src->SIGNAL(tier::release, e2::form::proceed::detach, data_src);
+                                                                data_src->SIGNAL(tier::anycast, e2::form::quit, data_src);
                                                                 gear.dismiss();
                                                             }
                                                         };
@@ -98,27 +105,45 @@ namespace netxs::app::desk
         };
         auto apps_template = [](auto& data_src, auto& apps_map)
         {
-            const static auto c3 = app::shared::c3;
-            const static auto x3 = app::shared::x3;
+            auto highlight_color = skin::color(tone::highlight);
+            auto inactive_color  = skin::color(tone::inactive);
+            auto c3 = highlight_color;
+            auto x3 = cell{ c3 }.alpha(0x00);
+            auto cA = inactive_color;
 
             auto apps = ui::list::ctor()
                 ->invoke([&](auto& boss)
                 {
                     boss.SUBMIT(tier::release, e2::form::upon::vtree::attached, parent)
                     {
-                        auto current_default = decltype(e2::data::changed)::type{};
+                        auto current_default = e2::data::changed.param();
                         boss.template riseup<tier::request>(e2::data::changed, current_default); //todo "template" required by gcc (ubuntu 18.04)
                         boss.SIGNAL(tier::anycast, events::ui::selected, current_default);
                     };
                 });
 
+            auto& conf_list = app::shared::get::configs();
+            auto def_note = text{" Menu item:                           \n"
+                                 "   Left click to start a new instance \n"
+                                 "   Right click to set default app     "};
             for (auto const& [class_id, stat_inst_ptr_list] : *apps_map)
             {
                 auto& [state, inst_ptr_list] = stat_inst_ptr_list;
-                auto inst_id  = class_id;
-                auto obj_desc = app::shared::objs_config[class_id].label;
+                auto inst_id = class_id;
+                auto& conf = conf_list[class_id];
+                auto& obj_desc = conf.label;
+                auto& obj_note = conf.notes;
+                if (conf.splitter)
+                {
+                    auto item_area = apps->attach(ui::pads::ctor(dent{ 0,0,0,1 }, dent{ 0,0,1,0 }))
+                                         ->attach(ui::item::ctor(obj_desc, true, faux, true))
+                                         ->colors(cA.fgc(), cA.bgc())
+                                         ->template plugin<pro::notes>(obj_note);
+                    continue;
+                }
                 auto item_area = apps->attach(ui::pads::ctor(dent{ 0,0,0,1 }, dent{ 0,0,1,0 }))
-                                     ->template plugin<pro::fader>(x3, c3, 0ms)
+                                     ->template plugin<pro::fader>(x3, c3, skin::timeout(tone::fastfader))
+                                     ->template plugin<pro::notes>(obj_note.empty() ? def_note : obj_note)
                                      ->invoke([&](auto& boss)
                                      {
                                          boss.mouse.take_all_events(faux);
@@ -137,16 +162,17 @@ namespace netxs::app::desk
                                              {
                                                  boss->SIGNAL(tier::anycast, events::ui::selected, inst_id);
                                              }
-                                             auto world_ptr = decltype(e2::config::whereami)::type{};
+                                             auto world_ptr = e2::config::whereami.param();
                                              SIGNAL_GLOBAL(e2::config::whereami, world_ptr);
                                              if (world_ptr)
                                              {
-                                                 static iota random = 0;
+                                                 static si32 random = 0;
                                                  random = (random + 2) % 10;
                                                  auto offset = twod{ random * 2, random };
                                                  auto viewport = gear.area();
                                                  gear.slot.coor = viewport.coor + viewport.size / 8 + offset;
                                                  gear.slot.size = viewport.size * 3 / 4;
+                                                 gear.slot_forced = faux;
                                                  world_ptr->SIGNAL(tier::release, e2::form::proceed::createby, gear);
                                                  gear.dismiss();
                                              }
@@ -177,8 +203,7 @@ namespace netxs::app::desk
                                          //           inst.SIGNAL(tier::preview, e2::form::layout::expose, inst);
                                          //           auto& area = inst.base::area();
                                          //           auto center = area.coor + (area.size / 2);
-                                         //           bell::getref(gear.id)->
-                                         //           SIGNAL(tier::release, e2::form::layout::shift, center);  // Goto to the window.
+                                         //           gear.owner.SIGNAL(tier::release, e2::form::layout::shift, center);  // Goto to the window.
                                          //           gear.pass_kb_focus(inst);
                                          //           gear.dismiss();
                                          //       }
@@ -197,37 +222,39 @@ namespace netxs::app::desk
                                     auto selected = inst_id == data;
                                     if (auto boss = boss_shadow.lock())
                                     {
-                                        boss->set(ansi::fgc4(selected ? 0xFF00ff00 : 0x00000000).add(obj_desc));
+                                        boss->set(ansi::fgx(selected ? 0xFF00ff00 : 0x00000000).add(obj_desc));
                                         boss->deface();
                                     }
                                 };
                             });
                     auto list_pads = block->attach(slot::_2, ui::pads::ctor(dent{ 0,0,0,0 }, dent{ 0,0,0,0 }));
                     auto insts = list_pads->attach(ui::list::ctor())
-                                          ->attach_collection(e2::form::prop::header, inst_ptr_list, app_template);
+                                          ->attach_collection(e2::form::prop::ui::header, inst_ptr_list, app_template);
             }
             return apps;
         };
 
-        auto build = [](view v)
+        auto build = [](text cwd, text v, xml::settings& config)
         {
-            iota uibar_min_size = 4;
-            iota uibar_full_size = 31;
+            auto lock = netxs::events::sync{}; // Protect access to the world.
+
+            auto uibar_min_size = si32{ 4  };
+            auto uibar_max_size = si32{ 31 };
 
             auto window = ui::cake::ctor();
 
             auto my_id = id_t{};
 
             auto user_info = utf::divide(v, ";");
-            if (user_info.size() < 3) return window;
+            if (user_info.size() < 2)
+            {
+                log("desk: bad window arguments: args=", utf::debase(v));
+                return window;
+            }
             auto& user_id___view = user_info[0];
             auto& user_name_view = user_info[1];
-            auto& user_path_view = user_info[2];
-
-            log("desk: user_id=", user_id___view, " user_name=", user_name_view, " user_path=", user_path_view);
-
-            auto user = text{ user_name_view };
-            auto path = text{ user_path_view };
+            auto& menu_selected  = user_info[2];
+            log("desk: id: ", user_id___view, ", user name: ", user_name_view);
 
             if (auto value = utf::to_int(user_id___view)) my_id = value.value();
             else return window;
@@ -238,13 +265,15 @@ namespace netxs::app::desk
                 auto client_shadow = ptr::shadow(client);
                 auto user_template = [my_id](auto& data_src, auto const& utf8)
                 {
-                    const static auto c3 = app::shared::c3;
-                    const static auto x3 = app::shared::x3;
+                    auto highlight_color = skin::color(tone::highlight);
+                    auto c3 = highlight_color;
+                    auto x3 = cell{ c3 }.alpha(0x00);
 
                     auto item_area = ui::pads::ctor(dent{ 1,0,0,1 }, dent{ 0,0,1,0 })
-                                            ->plugin<pro::fader>(x3, c3, 150ms);
+                                            ->plugin<pro::fader>(x3, c3, skin::timeout(tone::fader))
+                                            ->plugin<pro::notes>(" Connected user ");
                         auto user = item_area->attach(ui::item::ctor(ansi::esc(" &").nil().add(" ")
-                                    .fgc4(data_src->id == my_id ? rgba::color256[whitelt] : 0x00).add(utf8), true));
+                                    .fgx(data_src->id == my_id ? rgba::color256[whitelt] : 0x00).add(utf8), true));
                     return item_area;
                 };
                 auto branch_template = [user_template](auto& data_src, auto& usr_list)
@@ -254,15 +283,15 @@ namespace netxs::app::desk
                     return users;
                 };
 
-                window->invoke([uibar_full_size, uibar_min_size](auto& boss) mutable
+                window->invoke([uibar_max_size, uibar_min_size, menu_selected](auto& boss) mutable
                     {
-                        #ifdef _WIN32
-                            auto current_default_sptr = std::make_shared<text>(app::shared::objs_lookup["CommandPrompt"]);
-                            //auto current_default = app::shared::objs_lookup["PowerShell"];
-                        #else
-                            auto current_default_sptr = std::make_shared<text>(app::shared::objs_lookup["Term"]);
-                        #endif
-                        auto previous_default_sptr = std::make_shared<text>(*current_default_sptr);
+                        // Always set the first menu item as active.
+                        auto world_ptr = e2::config::whereami.param();
+                        SIGNAL_GLOBAL(e2::config::whereami, world_ptr);
+                        auto menu_list_ptr = e2::bindings::list::apps.param();
+                        world_ptr->SIGNAL(tier::request, e2::bindings::list::apps, menu_list_ptr);
+                        auto current_default_sptr  = std::make_shared<text>(menu_selected);
+                        auto previous_default_sptr = std::make_shared<text>(menu_selected);
                         auto subs_sptr = std::make_shared<subs>();
                         auto shadow = ptr::shadow(boss.This());
 
@@ -315,25 +344,30 @@ namespace netxs::app::desk
                                             });
                     auto taskbar = taskbar_viewport->attach(slot::_1, ui::fork::ctor(axis::Y))
                                         ->colors(whitedk, 0x60202020)
+                                        ->plugin<pro::notes>(" LeftDrag to adjust menu width                   \n"
+                                                             " Ctrl+LeftDrag to adjust folded width            \n"
+                                                             " RightDrag or scroll wheel to slide menu up/down ")
                                         ->plugin<pro::limit>(twod{ uibar_min_size,-1 }, twod{ uibar_min_size,-1 })
                                         ->plugin<pro::timer>()
                                         ->plugin<pro::acryl>()
                                         ->plugin<pro::cache>()
                                         ->invoke([&](auto& boss)
                                         {
-                                            boss.mouse.template draggable<sysmouse::left>();
+                                            boss.mouse.template draggable<sysmouse::left>(true);
                                             auto boss_shadow = ptr::shadow(boss.This());
-                                            auto size_config = std::make_shared<std::pair<iota, iota>>(uibar_full_size, uibar_min_size);
+                                            auto size_config = std::make_shared<std::pair<si32, si32>>(uibar_max_size, uibar_min_size);
                                             boss.SUBMIT_BYVAL(tier::release, e2::form::drag::pull::_<sysmouse::left>, gear)
                                             {
                                                 if (auto boss_ptr = boss_shadow.lock())
                                                 {
                                                     auto& boss = *boss_ptr;
-                                                    auto& [uibar_full_size, uibar_min_size] = *size_config;
+                                                    auto& [uibar_max_size, uibar_min_size] = *size_config;
                                                     auto& limits = boss.template plugins<pro::limit>();
                                                     auto lims = limits.get();
                                                     lims.min.x += gear.delta.get().x;
-                                                    lims.max.x = uibar_full_size = lims.min.x;
+                                                    lims.max.x = lims.min.x;
+                                                    gear.meta(hids::anyCtrl) ? uibar_min_size = lims.min.x
+                                                                             : uibar_max_size = lims.min.x;
                                                     limits.set(lims.min, lims.max);
                                                     boss.base::reflow();
                                                 }
@@ -347,9 +381,10 @@ namespace netxs::app::desk
                                                         if (auto boss_ptr = boss_shadow.lock())
                                                         {
                                                             auto& boss = *boss_ptr;
-                                                            auto& [uibar_full_size, uibar_min_size] = *size_config;
+                                                            auto& [uibar_max_size, uibar_min_size] = *size_config;
                                                             auto& limits = boss.template plugins<pro::limit>();
-                                                            auto size = active ? uibar_full_size : std::min(uibar_full_size, uibar_min_size);
+                                                            auto size = active ? std::max(uibar_max_size, uibar_min_size)
+                                                                               : uibar_min_size;
                                                             auto lims = twod{ size,-1 };
                                                             limits.set(lims, lims);
                                                             boss.base::reflow();
@@ -359,24 +394,28 @@ namespace netxs::app::desk
                                                     auto& timer = boss_ptr->template plugins<pro::timer>();
                                                     timer.pacify(faux);
                                                     if (active) apply(true);
-                                                    else timer.actify(faux, MENU_TIMEOUT, apply);
+                                                    else        timer.actify(faux, MENU_TIMEOUT, apply);
                                                 }
                                             };
                                             boss.SUBMIT_BYVAL(tier::anycast, e2::form::prop::viewport, viewport)
                                             {
-                                                auto& [uibar_full_size, uibar_min_size] = *size_config;
+                                                auto& [uibar_max_size, uibar_min_size] = *size_config;
                                                 viewport.coor.x += uibar_min_size;
                                                 viewport.size.x -= uibar_min_size;
                                             };
                                         });
                         auto apps_users = taskbar->attach(slot::_1, ui::fork::ctor(axis::Y, 0, 100));
                         {
-                            const static auto c3 = app::shared::c3;
-                            const static auto x3 = app::shared::x3;
-                            const static auto c6 = app::shared::c6;
-                            const static auto x6 = app::shared::x6;
+                            auto highlight_color = skin::color(tone::highlight);
+                            auto action_color    = skin::color(tone::action);
+                            auto inactive_color  = skin::color(tone::inactive);
+                            auto cA = inactive_color;
+                            auto c3 = highlight_color;
+                            auto x3 = cell{ c3 }.alpha(0x00);
+                            auto c6 = action_color;
+                            auto x6 = cell{ c6 }.alpha(0x00);
 
-                            auto world_ptr = decltype(e2::config::whereami)::type{};
+                            auto world_ptr = e2::config::whereami.param();
                             SIGNAL_GLOBAL(e2::config::whereami, world_ptr);
                             {
                                 auto applist_area = apps_users->attach(slot::_1, ui::pads::ctor(dent{ 0,0,1,0 }, dent{}))
@@ -391,13 +430,16 @@ namespace netxs::app::desk
                             {
                                 auto users_area = apps_users->attach(slot::_2, ui::fork::ctor(axis::Y));
                                 auto label_pads = users_area->attach(slot::_1, ui::pads::ctor(dent{ 0,0,1,1 }, dent{ 0,0,0,0 }))
-                                                            ->plugin<pro::fader>(x3, c3, 150ms);
+                                                            ->plugin<pro::notes>(" List of connected users ");
                                     auto label_bttn = label_pads->attach(ui::fork::ctor());
                                         auto label = label_bttn->attach(slot::_1,
-                                            ui::item::ctor(ansi::fgc(whitelt).add("Users"), faux, faux));
+                                            ui::item::ctor("users", true, faux, true))
+                                                ->plugin<pro::limit>(twod{ 5,-1 })
+                                                ->colors(cA.fgc(), cA.bgc());
                                         auto bttn_area = label_bttn->attach(slot::_2, ui::fork::ctor());
                                             auto bttn_pads = bttn_area->attach(slot::_2, ui::pads::ctor(dent{ 2,2,0,0 }, dent{ 0,0,1,1 }))
-                                                                      ->plugin<pro::fader>(x6, c6, 150ms);
+                                                                      ->plugin<pro::fader>(x6, c6, skin::timeout(tone::fader))
+                                                                      ->plugin<pro::notes>(" Show/hide user list ");
                                                 auto bttn = bttn_pads->attach(ui::item::ctor("<", faux));
                                 auto userlist_area = users_area->attach(slot::_2, ui::pads::ctor())
                                                                ->plugin<pro::limit>();
@@ -429,44 +471,40 @@ namespace netxs::app::desk
                             }
                         }
                         {
-                            const static auto c2 = app::shared::c2;
-                            const static auto x2 = app::shared::x2;
-                            const static auto c1 = app::shared::c1;
-                            const static auto x1 = app::shared::x1;
+                            auto warning_color = skin::color(tone::warning);
+                            auto danger_color  = skin::color(tone::danger);
+                            auto c2 = warning_color;
+                            auto x2 = cell{ c2 }.bga(0x00);
+                            auto c1 = danger_color;
+                            auto x1 = cell{ c1 }.bga(0x00);
 
                             auto bttns_cake = taskbar->attach(slot::_2, ui::cake::ctor());
                             auto bttns_area = bttns_cake->attach(ui::rail::ctor(axes::X_ONLY))
                                                         ->plugin<pro::limit>(twod{ -1, 3 }, twod{ -1, 3 });
                                 bttns_cake->attach(app::shared::underlined_hz_scrollbars(bttns_area));
                             auto bttns = bttns_area->attach(ui::fork::ctor(axis::X))
-                                                   ->plugin<pro::limit>(twod{ uibar_full_size, 3 }, twod{ -1, 3 });
+                                                   ->plugin<pro::limit>(twod{ uibar_max_size, 3 }, twod{ -1, 3 });
                                 auto disconnect_park = bttns->attach(slot::_1, ui::park::ctor())
-                                                            ->plugin<pro::fader>(x2, c2, 150ms)
+                                                            ->plugin<pro::fader>(x2, c2, skin::timeout(tone::fader))
+                                                            ->plugin<pro::notes>(" Leave current session ")
                                                             ->invoke([&](auto& boss)
                                                             {
                                                                 boss.SUBMIT(tier::release, hids::events::mouse::button::click::left, gear)
                                                                 {
-                                                                    if (auto owner = base::getref(gear.id))
-                                                                    {
-                                                                        owner->SIGNAL(tier::release, e2::conio::quit, "taskbar: logout by button");
-                                                                        gear.dismiss();
-                                                                    }
+                                                                    gear.owner.SIGNAL(tier::release, e2::conio::quit, "taskbar: logout by button");
+                                                                    gear.dismiss();
                                                                 };
                                                             });
                                 auto disconnect_area = disconnect_park->attach(snap::head, snap::center, ui::pads::ctor(dent{ 2,3,1,1 }));
                                 auto disconnect = disconnect_area->attach(ui::item::ctor("× Disconnect"));
                                 auto shutdown_park = bttns->attach(slot::_2, ui::park::ctor())
-                                                          ->plugin<pro::fader>(x1, c1, 150ms)
+                                                          ->plugin<pro::fader>(x1, c1, skin::timeout(tone::fader))
+                                                          ->plugin<pro::notes>(" Disconnect all users and shutdown the server ")
                                                           ->invoke([&](auto& boss)
                                                           {
                                                               boss.SUBMIT(tier::release, hids::events::mouse::button::click::left, gear)
                                                               {
-                                                                  //todo unify, see system.h:1614
-                                                                  #if defined(__APPLE__) || defined(__FreeBSD__)
-                                                                  auto path2 = "/tmp/" + path + ".sock";
-                                                                  ::unlink(path2.c_str());
-                                                                  #endif
-                                                                  os::exit(0, "taskbar: shutdown by button");
+                                                                  SIGNAL_GLOBAL(e2::shutdown, "desk: server shutdown");
                                                               };
                                                           });
                                 auto shutdown_area = shutdown_park->attach(snap::tail, snap::center, ui::pads::ctor(dent{ 2,3,1,1 }));
@@ -477,7 +515,7 @@ namespace netxs::app::desk
         };
     }
 
-    app::shared::initialize builder{ "Desk", build };
+    app::shared::initialize builder{ "desk", build };
 }
 
 #endif // NETXS_APP_DESK_HPP
\ No newline at end of file
