@@ -10,6 +10,7 @@
 #include <cmath>
 #include <cfenv>
 #include <cassert>
+#include <bit>
 
 #ifndef faux
     #define faux (false)
@@ -19,19 +20,157 @@ namespace netxs
 {
     using byte = uint8_t;
     using ui16 = uint16_t;
-    using iota = int32_t;
     using ui32 = uint32_t;
     using ui64 = uint64_t;
+    using si16 = int16_t;
+    using si32 = int32_t;
+    using si64 = int64_t;
+    using sz_t = ui32;
 
-    static constexpr iota maxiota = std::numeric_limits<iota>::max();
+    static constexpr auto maxsi32 = std::numeric_limits<si32>::max();
 
     constexpr size_t operator "" _sz (unsigned long long i)	{ return i; }
 
     struct noop { template<class ...T> constexpr void operator()(T...) {}; };
 
-    template <class T>
-    using to_signed_t = std::conditional_t<(int64_t)std::numeric_limits<std::remove_reference_t<T>>::max() <= std::numeric_limits<int16_t>::max(), int16_t,
-                        std::conditional_t<(int64_t)std::numeric_limits<std::remove_reference_t<T>>::max() <= std::numeric_limits<int32_t>::max(), int32_t, int64_t>>;
+    template<class T>
+    using to_signed_t = std::conditional_t<(si64)std::numeric_limits<std::remove_reference_t<T>>::max() <= std::numeric_limits<si16>::max(), si16,
+                        std::conditional_t<(si64)std::numeric_limits<std::remove_reference_t<T>>::max() <= std::numeric_limits<si32>::max(), si32, si64>>;
+
+    // intmath: Swap two bits.
+    template<unsigned int p1, unsigned int p2, class T>
+    auto swap_bits(T n)
+    {
+        auto a = 1 & (n >> p1);
+        auto b = 1 & (n >> p2);
+        auto x = a ^ b;
+        return n ^ (x << p1 | x << p2);
+    }
+    // intmath: Convert LE to host endianness.
+    template<class T>
+    constexpr void letoh(byte* buff, T& i)
+    {
+        if constexpr (std::is_same_v<T, ui32>
+                   || std::is_same_v<T, si32>)
+        {
+            i = (ui32)buff[0] <<  0 |
+                (ui32)buff[1] <<  8 |
+                (ui32)buff[2] << 16 |
+                (ui32)buff[3] << 24;
+        }
+        else if constexpr (std::is_same_v<T, ui64>
+                        || std::is_same_v<T, si64>)
+        {
+            i = (ui64)buff[0] <<  0 |
+                (ui64)buff[1] <<  8 |
+                (ui64)buff[2] << 16 |
+                (ui64)buff[3] << 24 |
+                (ui64)buff[4] << 32 |
+                (ui64)buff[5] << 40 |
+                (ui64)buff[6] << 48 |
+                (ui64)buff[7] << 56;
+        }
+        else if constexpr (std::is_same_v<T, ui16>
+                        || std::is_same_v<T, si16>)
+        {
+            i = (ui16)buff[0] << 0 |
+                (ui16)buff[1] << 8;
+        }
+    }
+    // intmath: Convert BE to host endianness.
+    template<class T>
+    constexpr void betoh(byte* buff, T& i)
+    {
+        if constexpr (std::is_same_v<T, ui32>
+                   || std::is_same_v<T, si32>)
+        {
+            i = (ui32)buff[3] <<  0 |
+                (ui32)buff[2] <<  8 |
+                (ui32)buff[1] << 16 |
+                (ui32)buff[0] << 24;
+        }
+        else if constexpr (std::is_same_v<T, ui64>
+                        || std::is_same_v<T, si64>)
+        {
+            i = (ui64)buff[7] <<  0 |
+                (ui64)buff[6] <<  8 |
+                (ui64)buff[5] << 16 |
+                (ui64)buff[4] << 24 |
+                (ui64)buff[3] << 32 |
+                (ui64)buff[2] << 40 |
+                (ui64)buff[1] << 48 |
+                (ui64)buff[0] << 56;
+        }
+        else if constexpr (std::is_same_v<T, ui16>
+                        || std::is_same_v<T, si16>)
+        {
+            i = (ui16)buff[1] << 0 |
+                (ui16)buff[0] << 8;
+        }
+    }
+    namespace
+    {
+        // intmath: Invert endianness.
+        template<class T>
+        constexpr auto _swap_bytes(T i)
+        {
+            T r;
+            if constexpr (std::is_same_v<T, ui32>
+                       || std::is_same_v<T, si32>)
+            {
+                auto n = static_cast<ui32>(i);
+                r = (n & 0x000000FF) << 24 |
+                    (n & 0x0000FF00) <<  8 |
+                    (n & 0x00FF0000) >>  8 |
+                    (n & 0xFF000000) >> 24;
+            }
+            else if constexpr (std::is_same_v<T, ui64>
+                            || std::is_same_v<T, si64>)
+            {
+                auto n = static_cast<ui64>(i);
+                r = (n & 0x00000000000000FF) << 56 |
+                    (n & 0x000000000000FF00) << 40 |
+                    (n & 0x0000000000FF0000) << 24 |
+                    (n & 0x00000000FF000000) <<  8 |
+                    (n & 0x000000FF00000000) >>  8 |
+                    (n & 0x0000FF0000000000) >> 24 |
+                    (n & 0x00FF000000000000) >> 40 |
+                    (n & 0xFF00000000000000) >> 56;
+            }
+            else if constexpr (std::is_same_v<T, ui16>
+                            || std::is_same_v<T, si16>)
+            {
+                auto n = static_cast<ui16>(i);
+                r = (n & 0x00FF) << 8 |
+                    (n & 0xFF00) >> 8;
+            }
+            return r;
+        }
+    }
+    // intmath: Convert LE to host endianness.
+    template<class T, bool BE = std::endian::native == std::endian::big>
+    constexpr auto letoh(T i)
+    {
+        if constexpr (BE) return _swap_bytes(i);
+        else              return i;
+    }
+    // intmath: Convert BE to host endianness.
+    template<class T, bool LE = std::endian::native == std::endian::little>
+    constexpr auto betoh(T i)
+    {
+        if constexpr (LE) return _swap_bytes(i);
+        else              return i;
+    }
+    // intmath: LE type wrapper. T has an LE format in memory.
+    template<class T>
+    class le_t
+    {
+        T data = {};
+
+    public:
+        void set(T const& v) { data = netxs::letoh(v); }
+        T    get() const     { return netxs::letoh(data); }
+    };
 
     // intmath: Summ and return TRUE in case of
     //          unsigned integer overflow and store result in accum.
@@ -113,22 +252,21 @@ namespace netxs
     template<class T>
     using disintegrate = typename _disintegrate< std::is_integral<T>::value, T >::type;
 
-    // intmath: Delta sequence generator.
+    // intmath: Quadratic fader delta sequence generator.
     //          The QUADRATIC-LAW fader from the initial velocity
     //          to stop for a given period of time.
     template<class T>
     class quadratic
     {
         using twod = T;
-        using iota = disintegrate<twod>;
+        using type = disintegrate<twod>;
 
-        twod speed;
-        iota limit;
-        iota phase;
-        iota scale;
-        iota start; // quadratic: Deffered start time.
-
-        mutable twod total;
+                twod speed; // quadratic: Distance ΔR over time period ΔT.
+                type limit; // quadratic: Activity period.
+                type phase; // quadratic: Register.
+                type scale; // quadratic: Factor.
+                type start; // quadratic: Deffered start time.
+        mutable twod total; // quadratic: Current point on the path.
 
     public:
         /*
@@ -138,7 +276,7 @@ namespace netxs
             limit - activity period
             start - deffered start time
         */
-        quadratic(twod const& speed, iota cycle, iota limit, iota start)
+        quadratic(twod const& speed, type cycle, type limit, type start)
             :	speed{ speed         },
                 limit{ limit         },
                 phase{ limit * 2     },
@@ -147,9 +285,9 @@ namespace netxs
                 total{ twod{}        }
         { }
 
-        auto operator ()(iota timer) const
+        auto operator () (type timer) const
         {
-            std::optional<twod> delta;
+            auto delta = std::optional<twod>{};
 
             if (auto t = timer - start; t >= 0)
             {
@@ -170,22 +308,21 @@ namespace netxs
         }
     };
 
-    // intmath: Delta sequence generator.
+    // intmath: Constant speed delta sequence generator.
     //          The LINEAR-LAW fader from the initial velocity
     //          to stop for a given period of time with constant speed.
     template<class T>
     class constlinear
     {
         using twod = T;
-        using iota = disintegrate<twod>;
+        using type = disintegrate<twod>;
 
-        iota limit;
-        iota phase;
-        twod speed;
-        iota scale;
-        iota start; // constlinear: Deffered start time.
-
-        mutable twod total;
+                type limit; // constlinear: Activity period.
+                type phase; // constlinear: Register.
+                twod speed; // constlinear: Distance ΔR over time period ΔT.
+                type scale; // constlinear: Factor.
+                type start; // constlinear: Deffered start time.
+        mutable twod total; // constlinear: Current point on the path.
 
     public:
         /*
@@ -195,7 +332,7 @@ namespace netxs
             limit - activity period
             start - deffered start time
         */
-        constlinear(twod const& speed, iota cycle, iota limit, iota start)
+        constlinear(twod const& speed, type cycle, type limit, type start)
             :	limit{ limit         },
                 phase{ limit * 2     },
                 speed{ speed * phase },
@@ -204,9 +341,9 @@ namespace netxs
                 total{ twod{}        }
         { }
 
-        auto operator ()(iota timer) const
+        auto operator () (type timer) const
         {
-            std::optional<twod> delta;
+            auto delta = std::optional<twod>{};
 
             if (auto t = timer - start; t >= 0)
             {
@@ -227,38 +364,31 @@ namespace netxs
         }
     };
 
-    // intmath: Delta sequence generator.
-    //          The LINEAR-LAW fader from the initial coord to the destination
-    //          coord for a given period of time with constant speed.
+    // intmath: Constant speed delta sequence generator.
+    //          The LINEAR-LAW fader from the initial point to the destination
+    //          point for a given period of time with constant speed.
     template<class T>
     class constlinearAtoB
     {
         using twod = T;
-        using iota = disintegrate<twod>;
+        using type = disintegrate<twod>;
 
-        iota limit;
-        twod range;
-        iota start; // constlinear: Deffered start time.
-
-        mutable twod total;
+                type limit; // constlinearAtoB: Activity period.
+                twod range; // constlinearAtoB: Path's end point (from 0 to range).
+                type start; // constlinearAtoB: Deffered start time point.
+        mutable twod total; // constlinearAtoB: Current point on the path.
 
     public:
-        /*
-        Linear constant speed delta generator ctor:
-        range - all path (from 0 to range)
-        limit - activity period
-        start - deffered start time
-        */
-        constlinearAtoB(twod const& range, iota limit, iota start)
+        constlinearAtoB(twod const& range, type limit, type start)
             :	limit{ limit },
                 range{ range },
                 start{ start },
                 total{ twod{}}
         { }
 
-        auto operator ()(iota timer) const
+        auto operator () (type timer) const
         {
-            std::optional<twod> delta;
+            auto delta = std::optional<twod>{};
 
             if (auto t = timer - start; t >= 0)
             {
@@ -304,6 +434,47 @@ namespace netxs
         return width;
     }
 
+    // intmath: Fill the canvas by the stretched bitmap.
+    template<class T, class P>
+    void zoomin(T& canvas, T const& bitmap, P handle)
+    {
+        auto size1 = canvas.size();
+        auto size2 = bitmap.size();
+        auto data1 = canvas.data();
+        auto data2 = bitmap.data();
+        auto data3 = data2;
+        if (size1.x * size1.y == 0
+         || size2.x * size2.y == 0) return;
+
+        auto dot_11 = size1 / size1;
+        auto msize0 = size1 - dot_11;
+        auto msize1 = max(dot_11, msize0);
+        auto msize2 = size2 - dot_11;
+
+        auto y = 0;
+        auto h_line = [&]
+        {
+            auto x = 0;
+            while (x != msize0.x)
+            {
+                auto xpos = x++ * msize2.x / msize1.x;
+                auto from = data3 + xpos;
+                handle(*data1++, *from);
+            }
+            auto ypos = ++y * msize2.y / msize1.y;
+            auto from = data3 + msize2.x;
+            handle(*data1++, *from);
+            data3 = data2 + ypos * size2.x;
+        };
+
+        while (y != msize0.y)
+        {
+            h_line();
+        }
+        data3 = data2 + msize2.y * size2.x;
+        h_line();
+    }
+
     // intmath: Copy the bitmap to the bitmap by invoking
     //          handle(sprite1_element, sprite2_element) for each elem.
     template<class T, class P>
@@ -328,8 +499,8 @@ namespace netxs
     // intmath: Intersect two sprites and
     //          invoking handle(sprite1_element, sprite2_element)
     //          for each elem in the intersection.
-    template<bool RtoL, class T, class R, class C, class P>
-    void inbody(T& canvas, T const& bitmap, R const& region, C const& base2, P handle)
+    template<bool RtoL, class T, class D, class R, class C, class P>
+    void inbody(T& canvas, D const& bitmap, R const& region, C const& base2, P handle)
     {
         auto& base1 = region.coor;
 
@@ -365,11 +536,29 @@ namespace netxs
         }
     }
 
+    // intmath: Wild bitmap.
+    template<class T, class Rect>
+    struct raster
+    {
+        T    _data;
+        Rect _area;
+        auto  data()       { return _data.begin(); }
+        auto& size()       { return _area.size;    }
+        auto& area()       { return _area;         }
+        auto  data() const { return _data.begin(); }
+        auto& size() const { return _area.size;    }
+        auto& area() const { return _area;         }
+        raster(T data, Rect area)
+            : _data{ data },
+              _area{ area }
+        { }
+    };
+
     // intmath: Intersect two sprites and invoking
     //          handle(sprite1_element, sprite2_element)
     //          for each elem in the intersection.
-    template<class T, class P>
-    void onbody(T& canvas, T const& bitmap, P handle)
+    template<class T, class D, class P>
+    void onbody(T& canvas, D const& bitmap, P handle)
     {
         auto& rect1 = canvas.area();
         auto& rect2 = bitmap.area();
@@ -386,25 +575,37 @@ namespace netxs
     // intmath: Draw the rectangle region inside the canvas by
     //          invoking handle(canvas_element)
     //          (without boundary checking).
-    template<class T, class RECT, class P, class NEWLINEFX = noop>
+    template<bool RtoL = faux, class T, class RECT, class P, class NEWLINEFX = noop>
     void onrect(T& canvas, RECT const& region, P handle, NEWLINEFX online = NEWLINEFX())
     {
-        auto& place = canvas.area();
+        //using ret_t = std::template result_of_t<P(decltype(*(canvas.data())))>;
+        using ret_t = std::invoke_result_t<P, decltype(*(canvas.data()))>;
+        static constexpr auto plain = std::is_same_v<void, ret_t>;
 
+        auto& place = canvas.area();
         if (auto joint = region.clip(place))
         {
             auto basis = joint.coor - place.coor;
             auto frame = place.size.x * basis.y + basis.x + canvas.data();
             auto notch = place.size.x - joint.size.x;
-
             auto limit = place.size.x * joint.size.y + frame;
             while (limit != frame)
             {
                 auto limit = frame + joint.size.x;
                 while (limit != frame)
                 {
-                    handle(*frame++);
+                    if constexpr (RtoL)
+                    {
+                        if constexpr (plain) handle(*--limit);
+                        else             if (handle(*--limit)) return;
+                    }
+                    else
+                    {
+                        if constexpr (plain) handle(*frame++);
+                        else             if (handle(*frame++)) return;
+                    }
                 }
+                if constexpr (RtoL) frame += joint.size.x;
                 frame += notch;
                 online();
             }
@@ -420,8 +621,11 @@ namespace netxs
         auto left = x1 - xmin;
         auto top  = y1 - ymin;
 
-        if ((dx == 0.0f && left < 0.0f) ||
-            (dy == 0.0f && top  < 0.0f)) return faux; // Line is parallel to rectangle.
+        if ((dx == 0.0f && left < 0.0f)
+         || (dy == 0.0f && top  < 0.0f))
+        {
+            return faux; // Line is parallel to rectangle.
+        }
 
         auto max = 0.0f;
         auto min = 1.0f;
@@ -477,17 +681,17 @@ namespace netxs
     bool online(R rect, T p0, T p1, P pset)
     {
         using twod = T;
-        using iota = disintegrate<twod>;
+        using type = disintegrate<twod>;
 
-        uint16_t gain = 0;
-        int32_t  dx = p1.x - p0.x;
-        int32_t  dy = p1.y - p0.y;
-        uint32_t lx = std::abs(dx);
-        uint32_t ly = std::abs(dy);
+        auto gain = static_cast<ui16>( 0 );
+        auto   dx = static_cast<si32>( p1.x - p0.x );
+        auto   dy = static_cast<si32>( p1.y - p0.y );
+        auto   lx = static_cast<ui32>( std::abs(dx) );
+        auto   ly = static_cast<ui32>( std::abs(dy) );
 
         rect = rect.normalize();
-        twod& coor = rect.coor;
-        twod& size = rect.size;
+        auto& coor = rect.coor;
+        auto& size = rect.size;
         p0 -= coor;
         p1 -= coor;
 
@@ -544,22 +748,26 @@ namespace netxs
                 auto decr = [](auto& a){ --a; };
 
                 if (dy > 0)
+                {
                     if (dx > 0) line(p0, p1, incr, incr);
                     else        line(p0, p1, decr, incr);
+                }
                 else
+                {
                     if (dx > 0) line(p0, p1, incr, decr);
                     else        line(p0, p1, decr, decr);
+                }
             }
         };
 
         if (!size.inside(p0) || !size.inside(p1))
         {
-            float x1 = static_cast<float>(p0.x); float y1 = static_cast<float>(p0.y);
-            float x2 = static_cast<float>(p1.x); float y2 = static_cast<float>(p1.y);
-            float minx = -1.0f;// One element wide margin for antialiasing.
-            float miny = -1.0f;//
-            float maxx = size.x + 1.0f;
-            float maxy = size.y + 1.0f;
+            auto x1 = static_cast<float>(p0.x); auto y1 = static_cast<float>(p0.y);
+            auto x2 = static_cast<float>(p1.x); auto y2 = static_cast<float>(p1.y);
+            auto minx = -1.0f;// One element wide margin for antialiasing.
+            auto miny = -1.0f;//
+            auto maxx = size.x + 1.0f;
+            auto maxy = size.y + 1.0f;
 
             if (liang_barsky(minx, miny, maxx, maxy, x1, y1, x2, y2))
             {
@@ -573,10 +781,10 @@ namespace netxs
                     return delta;
                 };
 
-                p1.x = static_cast<iota>(x2);
-                p1.y = static_cast<iota>(y2);
-                auto new_x = static_cast<iota>(x1);
-                auto new_y = static_cast<iota>(y1);
+                p1.x = static_cast<type>(x2);
+                p1.y = static_cast<type>(y2);
+                auto new_x = static_cast<type>(x1);
+                auto new_y = static_cast<type>(y1);
 
                 if (p0.x != new_x || p0.y != new_y)
                 {
@@ -650,7 +858,7 @@ namespace netxs
             while (s_ptr < limit)
             {
                 auto& first = s_ref(s_ptr);
-                RGB_T accum = first;
+                auto  accum = RGB_T{ first };
                 accum *= rad_1;
 
                 auto front = s_ptr;
