@@ -7,6 +7,7 @@
 #include "../abstract/ptr.hpp"
 #include "../abstract/hash.hpp"
 #include "../abstract/duplet.hpp"
+#include "../abstract/queue.hpp"
 
 #include <vector>
 #include <mutex>
@@ -15,6 +16,7 @@
 #include <functional>
 #include <optional>
 #include <thread>
+#include <condition_variable>
 
 namespace netxs::events
 {
@@ -42,20 +44,24 @@ namespace netxs::events
     {
         std::lock_guard<std::recursive_mutex> lock;
 
-        sync            (sync const&) = delete; // deleted copy constructor.
-        sync& operator= (sync const&) = delete; // deleted copy assignment operator.
+        sync             (sync const&) = delete; // deleted copy constructor.
+        sync& operator = (sync const&) = delete; // deleted copy assignment operator.
 
         sync() : lock(_globals<void>::mutex) { }
        ~sync() { }
     };
+    static auto unique_lock()
+    {
+        return std::unique_lock{ _globals<void>::mutex };
+    }
     struct try_sync
     {
         std::unique_lock<std::recursive_mutex> lock;
 
-        operator bool() { return lock.owns_lock(); }
+        operator bool () { return lock.owns_lock(); }
 
-        try_sync            (try_sync const&) = delete; // deleted copy constructor.
-        try_sync& operator= (try_sync const&) = delete; // deleted copy assignment operator.
+        try_sync             (try_sync const&) = delete; // deleted copy constructor.
+        try_sync& operator = (try_sync const&) = delete; // deleted copy assignment operator.
 
         try_sync() : lock(_globals<void>::mutex, std::try_to_lock) { }
        ~try_sync() { }
@@ -79,7 +85,7 @@ namespace netxs::events
     static constexpr auto block = 4;
 
     // events: Return event level by its ID. Find the log base 2**block.
-    constexpr static inline auto level(type event)
+    static constexpr inline auto level(type event)
     {
         if (event == 0) return 0;
         auto level = 1;
@@ -87,7 +93,7 @@ namespace netxs::events
         return level;
     }
     // events: Return event level mask by its ID. Find the log base 2**block.
-    constexpr static inline type level_mask(type event)
+    static constexpr inline type level_mask(type event)
     {
         auto level = 0;
         while (event >>= block) { level += block; }
@@ -137,16 +143,22 @@ namespace netxs::events
                 : proc{ proc }
             { }
         };
+        enum branch
+        {
+            fullstop,
+            nothandled,
+            proceed,
+        };
 
         std::map<type, list> stock; // reactor: Handlers repository.
         std::vector<type>    queue; // reactor: Event queue.
         vect                 qcopy; // reactor: Copy of the current pretenders to exec on current event.
-        bool                 alive; // reactor: Current exec branch interruptor.
+        branch               alive; // reactor: Current exec branch interruptor.
 
         void cleanup(ui64& ref_count, ui64& del_count)
         {
-            ui64 lref{};
-            ui64 ldel{};
+            auto lref = ui64{};
+            auto ldel = ui64{};
             for (auto& [event, subs] : stock)
             {
                 auto refs = subs.size();
@@ -171,7 +183,7 @@ namespace netxs::events
         {
             auto proc_ptr = std::make_shared<wrapper<F>>(std::move(proc));
 
-            sync lock;
+            auto lock = sync{};
             stock[event].push_back(proc_ptr);
 
             return proc_ptr;
@@ -184,16 +196,16 @@ namespace netxs::events
         template<class F>
         auto notify(type event, F&& param)
         {
-            sync lock;
+            auto lock = sync{};
 
-            alive = true;
+            alive = branch::proceed;
             queue.push_back(event);
             auto head = qcopy.size();
 
             if constexpr (ORDER == execution_order::forward)
             {
-                type itermask = events::level_mask(event);
-                type subgroup = event;
+                auto itermask = events::level_mask(event);
+                auto subgroup = event;
                 _refreshandcopy(stock[subgroup]);
                 while (itermask > 1 << events::block) // Skip root event block.
                 {
@@ -204,9 +216,9 @@ namespace netxs::events
             }
             else
             {
-                static constexpr type mask = (1 << events::block) - 1;
-                type itermask = mask; // Skip root event block.
-                type subgroup;
+                static constexpr auto mask = type{ (1 << events::block) - 1 };
+                auto itermask = mask; // Skip root event block.
+                auto subgroup = type{};
                 do
                 {
                     itermask = itermask << events::block | mask;
@@ -231,17 +243,22 @@ namespace netxs::events
                         }
                     }
                 }
-                while (alive && ++iter != tail);
+                while (alive == branch::proceed && ++iter != tail);
                 qcopy.resize(head);
             }
 
             queue.pop_back();
-            return size;
+            return alive != branch::nothandled && size;
         }
         // reactor: Interrupt current invocation branch.
         void stop()
         {
-            alive = faux;
+            alive = branch::fullstop;
+        }
+        // reactor: Skip current invocation branch.
+        void skip()
+        {
+            alive = branch::nothandled;
         }
     };
 
@@ -256,12 +273,12 @@ namespace netxs::events
         // indexer: Return shared_ptr of the object by its id.
         static auto getref(id_t id)
         {
-            sync lock;
+            auto lock = sync{};
             return netxs::get_or(store, id, empty).lock();
         }
         // indexer: Create a new object of the specified subtype and return its shared_ptr.
         template<class TT, class ...Args>
-        static auto create(Args&&... args)
+        static auto create(Args&&... args) -> sptr<TT>
         {
             // Enables the use of a protected ctor by std::make_shared<TT>.
             struct make_shared_enabler : public TT
@@ -271,8 +288,8 @@ namespace netxs::events
                 { }
             };
 
-            sync lock;
-            sptr<TT> inst = std::make_shared<make_shared_enabler>(std::forward<Args>(args)...);
+            auto lock = sync{};
+            auto inst = std::make_shared<make_shared_enabler>(std::forward<Args>(args)...);
 
             store[inst->id] = inst;
             //sptr<T>  item = inst;
@@ -283,16 +300,23 @@ namespace netxs::events
     private:
         static inline auto _counter()
         {
-            sync lock;
-            while (netxs::on_key(store, ++newid)) { }
+            auto lock = sync{};
+            while (netxs::on_key(store, ++newid))
+            { }
             return newid;
         }
 
     protected:
         indexer(indexer const&) = delete; // id is flushed out when a copy of the object is deleted.
                                           // Thus, the original object instance becomes invalid.
-        indexer() : id { _counter() } { }
-       ~indexer() { sync lock; store.erase(id); }
+        indexer()
+            : id{ _counter() }
+        { }
+       ~indexer()
+        {
+           auto lock = sync{};
+           store.erase(id);
+        }
     };
 
     template<class T> id_t                    indexer<T>::newid{};
@@ -304,10 +328,12 @@ namespace netxs::events
         std::vector<hook> tokens;
 
     public:
+        operator bool () const     { return tokens.size();                                                   }
         void  admit(hook&& t)      {        tokens.push_back(std::forward<hook>(t));                         }
         hook& extra()              { return tokens.emplace_back();                                           }
         auto  count() const        { return tokens.size();                                                   }
         void  clear()              {        tokens.clear();                                                  }
+        void  reset()              {        tokens.clear();                                                  }
         void  merge(subs const& m) {        tokens.insert( tokens.end(), m.tokens.begin(), m.tokens.end() ); }
     };
 
@@ -318,8 +344,9 @@ namespace netxs::events
         using base = _parent_type;
         static constexpr auto id = _event_id;
         template<class ...Args> constexpr type_clue(Args&&...) { }
-        template<auto N> static constexpr auto group() { return events::subset<id, N>; }
-                         static constexpr auto index() { return events::number<id>;    }
+        template<class ...Args> static constexpr auto param(Args&&... args) { return type{ std::forward<Args>(args)... }; }
+        template<auto N>        static constexpr auto group()               { return events::subset<id, N>;               }
+                                static constexpr auto index()               { return events::number<id>;                  }
     };
 
     #define SUBMIT(        level, event,        param) bell::template submit<level, decltype( event )>()        = [&] (typename decltype( event )::type &&  param)
@@ -400,8 +427,16 @@ namespace netxs::events
             using type = typename EVENT::type;
             bell& owner;
             type& p;
-            submit_helper2(bell& owner, type& p) : owner{ owner }, p{p} { }
-            template<class F> void operator=(F h) { owner.submit<TIER, EVENT>(h); h(static_cast<type&&>(p));  }
+            submit_helper2(bell& owner, type& p)
+                : owner{ owner },
+                  p{p}
+            { }
+            template<class F>
+            void operator = (F h)
+            {
+                owner.submit<TIER, EVENT>(h);
+                h(static_cast<type&&>(p));
+            }
         };
         //todo deprecated?
         template<tier TIER, class EVENT>
@@ -416,29 +451,53 @@ namespace netxs::events
                   p{p},
                   token{ token }
             { }
-            template<class F> void operator=(F h) { owner.submit<TIER, EVENT>(token, h); h(static_cast<type&&>(p));  }
+            template<class F>
+            void operator = (F h)
+            {
+                owner.submit<TIER, EVENT>(token, h);
+                h(static_cast<type&&>(p));
+            }
         };
         template<tier TIER, class EVENT>
         struct submit_helper
         {
             bell& owner;
-            submit_helper(bell& owner) : owner{ owner } { }
-            template<class F> void operator=(F h) { owner.submit<TIER, EVENT>(h); }
+            submit_helper(bell& owner)
+                : owner{ owner }
+            { }
+            template<class F>
+            void operator = (F h)
+            {
+                owner.submit<TIER, EVENT>(h);
+            }
         };
         template<tier TIER, class EVENT>
         struct submit_helper_token
         {
             bell& owner;
             hook& token;
-            submit_helper_token(bell& owner, hook& token) : owner{ owner }, token{ token } { }
-            template<class F> void operator=(F h) { owner.submit<TIER, EVENT>(token, h); }
+            submit_helper_token(bell& owner, hook& token)
+                : owner{ owner },
+                  token{ token }
+            { }
+            template<class F>
+            void operator = (F h)
+            {
+                owner.submit<TIER, EVENT>(token, h);
+            }
         };
         template<class EVENT>
         struct submit_helper_token_global
         {
             hook& token;
-            submit_helper_token_global(hook& token) : token{ token } { }
-            template<class F> void operator=(F h) { token = _globals<void>::general.subscribe(EVENT::id, std::function<void(typename EVENT::type &&)>{ h }); }
+            submit_helper_token_global(hook& token)
+                : token{ token }
+            { }
+            template<class F>
+            void operator = (F h)
+            {
+                token = _globals<void>::general.subscribe(EVENT::id, std::function<void(typename EVENT::type &&)>{ h });
+            }
         };
 
     public:
@@ -530,10 +589,34 @@ namespace netxs::events
         }
        ~bell() { SIGNAL(tier::release, userland::root::dtor, id); }
 
-        virtual void  global(twod& coor) { } // bell: Recursively calculate global coordinate.
+        virtual void global(twod& coor) { } // bell: Recursively calculate global coordinate.
         virtual sptr<bell> gettop() { return sptr<bell>(this, noop{}); } // bell: Recursively find the root of the visual tree.
     };
 
+    namespace
+    {
+        template<class T>
+        auto& _agent()
+        {
+            static auto agent = netxs::jobs<netxs::wptr<bell>>{};
+            return agent;
+        }
+    }
+    template<class T>
+    void enqueue(netxs::wptr<bell> object_wptr, T&& proc)
+    {
+        auto& agent = _agent<void>();
+        agent.add(object_wptr, [proc](auto& object_wptr) mutable
+        {
+            auto lock = events::sync{};
+            if (auto object_ptr = object_wptr.lock())
+            {
+                proc(*object_ptr);
+            }
+        });
+
+    }
+
     template<class T> bell::fwd_reactor bell::_globals<T>::general;
 }
 
